// 
// Copyright (C) 2021, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using NinjaTrader.Cbi;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Tools;

#endregion

// This is a customized improved version of ninjatrader8 risk-reward tool
// Improved by LeighB (leighsdesk@pm.me) 2021-10-09 Rev2

#region Custom Enums
namespace CustomEnums
{
	public enum RiskPSDisplayStyleList
	{
		[Description("at beginning    1234 contracts @ price")]
		style1,
		[Description("at beginning    1234 lots @ price")]
		style2,
		[Description("at beginning    1234 @ price")]
		style3,
		[Description("at beginning    (1234 contracts)")]
		style4,
		[Description("at beginning    (1234 lots)")]
		style5,
		[Description("at beginning    (1234)")]
		style6,
		[Description("at end    1234 contracts")]
		style7,
		[Description("at end    1234 lots")]
		style8,
		[Description("at end    contracts 1234")]
		style9,
		[Description("at end    lots 1234")]
		style10,
		[Description("at end    (contracts 1234)")]
		style11,
		[Description("at end    (lots 1234)")]
		style12,
		[Description("at end    (1234 contracts)")]
		style13,
		[Description("at end    (1234 lots)")]
		style14,
		[Description("at end    (1234)")]
		style15,
	}

	public class EnumDescriptionConverter : EnumConverter
	{
		private Type enumType;
		
		public EnumDescriptionConverter(Type type) : base(type)
		{
			enumType = type;
		}

		public override bool CanConvertFrom(ITypeDescriptorContext context, Type srcType)
		{
			return srcType == typeof(string);
		}

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destType)
		{
			return destType == typeof(string);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			foreach (System.Reflection.FieldInfo fi in enumType.GetFields())
			{
				DescriptionAttribute dna = (DescriptionAttribute)Attribute.GetCustomAttribute(fi, typeof(DescriptionAttribute));
				if (dna != null && (string)value == dna.Description)
					return Enum.Parse(enumType, fi.Name);
			}
			return Enum.Parse(enumType, (string)value);
		}

		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destType)
		{
			System.Reflection.FieldInfo fi = enumType.GetField(value.ToString());
			DescriptionAttribute dna = (DescriptionAttribute) Attribute.GetCustomAttribute(fi, typeof(DescriptionAttribute));
			if(dna != null)
				return dna.Description;
			else
				return value.ToString();
		}

		public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
		{
			List<string> values = new List<string>();
			foreach (var v in Enum.GetValues(enumType))
			{
				System.Reflection.FieldInfo fi = enumType.GetField(v.ToString());
				DescriptionAttribute dna = (DescriptionAttribute)Attribute.GetCustomAttribute(fi, typeof(DescriptionAttribute));
				if (dna != null)
					values.Add(dna.Description);
				else
					values.Add(v.ToString());
			}
			return new StandardValuesCollection(values);
		}
	}
}
#endregion


//This namespace holds Drawing tools in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.DrawingTools
{
	/// <summary>
	/// Represents an interface that exposes information regarding a Risk Reward IDrawingTool.
	/// </summary>
	public class leighRiskRewardPositionSizing : DrawingTool
	{
		private const int		cursorSensitivity		= 15;
		private ChartAnchor		editingAnchor;
		private double			entryPrice;
		private bool			needsRatioUpdate		= true;
		private double			ratio1					= 0;
		private double			ratio2					= 0;
		private double			ratio3					= 0;
		private double			ratio4					= 0;
		private double			risk;
		private double			reward;	
		private double			reward2;	
		private double			reward3;	
		private double			reward4;
		private double			stopPrice;
		private double			targetPrice;
		private double			targetPrice2;
		private double			targetPrice3;
		private double			targetPrice4;
		private double			textleftPoint;
		private	double			textRightPoint;
		Account 				Risk_PS_Account;
        private double			Risk_PS_AccountBalance;
		int 					Risk_PS_contractquantity = 0;
		string 					Risk_PS_contractquantity_str;

	
		[Browsable(false)]
		private bool 			DrawTarget				{ get { return (RiskAnchor != null && !RiskAnchor.IsEditing) || (RewardAnchor1 != null && !RewardAnchor1.IsEditing); } }

		[Display(Order = 1)]
		public ChartAnchor		EntryAnchor				{ get; set; }

		[Display(Order = 2)]
		public ChartAnchor		RiskAnchor				{ get; set; }

		
		[Browsable(false)]
		public ChartAnchor		RewardAnchor1			{ get; set; }
		public ChartAnchor		RewardAnchor2			{ get; set; }
		public ChartAnchor		RewardAnchor3			{ get; set; }
		public ChartAnchor		RewardAnchor4			{ get; set; }

		public override object Icon { get { return Icons.DrawRiskReward; } }

		[Range(0, double.MaxValue)]
		[NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name="Reward ratio - Target 1", GroupName="NinjaScriptGeneral", Order = 1)]
		public double Ratio1
		{
			get { return ratio1; }
			set
			{
				if (ratio1.ApproxCompare(value) == 0)
					return;
				ratio1				= value;
				needsRatioUpdate	= true;
			}
		}
		[NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name="Reward ratio - Target 2", GroupName="NinjaScriptGeneral", Order = 2)]
		public double Ratio2
		{
			get { return ratio2; }
			set
			{
				if (ratio2.ApproxCompare(value) == 0)
					return;
				ratio2				= value;
				needsRatioUpdate	= true;
			}
		}
		[NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name="Reward ratio - Target 3", GroupName="NinjaScriptGeneral", Order = 3)]
		public double Ratio3
		{
			get { return ratio3; }
			set
			{
				if (ratio3.ApproxCompare(value) == 0)
					return;
				ratio3				= value;
				needsRatioUpdate	= true;
			}
		}
		[NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name="Reward ratio - Target 4", GroupName="NinjaScriptGeneral", Order = 4)]
		public double Ratio4
		{
			get { return ratio4; }
			set
			{
				if (ratio4.ApproxCompare(value) == 0)
					return;
				ratio4				= value;
				needsRatioUpdate	= true;
			}
		}
		
		[Display(ResourceType = typeof(Custom.Resource), Name = "NinjaScriptDrawingToolTextAlignment", GroupName="NinjaScriptGeneral", Order = 5)]
		public TextLocation				TextAlignment 			{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "Show text labels (entry, stop, T1 etc)", GroupName="NinjaScriptGeneral", Order = 6)]
		public bool bShowLabels	{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "NinjaScriptDrawingToolRulerYValueDisplayUnit", GroupName = "NinjaScriptGeneral", Order = 7)]
		public ValueUnit				DisplayUnit 		{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "NinjaScriptDrawingToolFibonacciRetracementsExtendLinesLeft", GroupName = "NinjaScriptLines", Order = 1)]
		public bool						IsExtendedLinesLeft 	{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "NinjaScriptDrawingToolFibonacciRetracementsExtendLinesRight", GroupName = "NinjaScriptLines", Order = 2)]
		public bool						IsExtendedLinesRight 	{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "Entry line", GroupName = "NinjaScriptLines", Order = 4)]
		public Stroke			EntryLineStroke 		{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "Risk / Stop line", GroupName = "NinjaScriptLines", Order = 5)]
		public Stroke			StopLineStroke 			{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "Reward / Target 1 line", GroupName = "NinjaScriptLines", Order = 6)]
		public Stroke			TargetLine1Stroke 		{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "Reward / Target 2 line", GroupName = "NinjaScriptLines", Order = 7)]
		public Stroke			TargetLine2Stroke 		{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "Reward / Target 3 line", GroupName = "NinjaScriptLines", Order = 8)]
		public Stroke			TargetLine3Stroke 		{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "Reward / Target 4 line", GroupName = "NinjaScriptLines", Order = 9)]
		public Stroke			TargetLine4Stroke 		{ get; set; }
		[Display(ResourceType = typeof(Custom.Resource), Name = "NinjaScriptDrawingToolAnchor", GroupName = "NinjaScriptLines", Order = 10)]
		public Stroke			AnchorLineStroke 		{ get; set; }
		public override IEnumerable<ChartAnchor> Anchors { get { return new[] { EntryAnchor, RiskAnchor, RewardAnchor1}; } }
		
		
		[NinjaScriptProperty]
        [Display(Name = "Show position sizing text", GroupName = "Position Sizing", Order = 1)]
        public bool bShow_PS { get; set; }
		private CustomEnums.RiskPSDisplayStyleList Risk_PS_Style;
		[NinjaScriptProperty]
		[TypeConverter(typeof(CustomEnums.EnumDescriptionConverter))]
		[PropertyEditor("NinjaTrader.Gui.Tools.StringStandardValuesEditorKey")]
		[Display(Name = "Position sizing text - format / appearance", GroupName = "Position Sizing", Order = 2)]
		public CustomEnums.RiskPSDisplayStyleList RiskPSStyle
		{
			get { return Risk_PS_Style; }
			set { Risk_PS_Style = value; }
		}
		[NinjaScriptProperty]
        [TypeConverter(typeof(NinjaTrader.NinjaScript.AccountNameConverter))]
        [Display(Name = "Account", GroupName = "Position Sizing", Order = 3)]
		[RefreshProperties(RefreshProperties.All)]
        public string Risk_PS_AccountName { get; set; }
		[NinjaScriptProperty]
        [Display(Name = "Risk % of account", GroupName = "Position Sizing", Order = 4)]
        public float Risk_PS_MaxPercentageOnAccount { get; set; }
		[NinjaScriptProperty]
        [Display(Name = "Show warning when risk exceeded", GroupName = "Position Sizing", Order = 5)]
        public bool bShowWarningExceeded { get; set; }
		

		public override bool SupportsAlerts { get { return true; } }

		private void DrawPriceText(ChartAnchor anchor, Point point, double price, ChartControl chartControl, ChartPanel chartPanel, ChartScale chartScale, String extratext)
		{
			if (TextAlignment == TextLocation.Off)
				return;
			
			string priceString;
			ChartBars chartBars = GetAttachedToChartBars();
			
			// bars can be null while chart is initializing
			if (chartBars == null)
				return;
			
			// NS can change ChartAnchor price via Draw method or directly meaning we needed to resync price before drawing
			if (!IsUserDrawn)
				price = AttachedTo.Instrument.MasterInstrument.RoundToTickSize(anchor.Price);

			
			// default
			priceString = GetPriceString(price, chartBars);

			if (!bShowLabels)
			{
				extratext = "";
			}

			
			if (bShow_PS)
			{
				switch(	Risk_PS_Style)
				{
					case CustomEnums.RiskPSDisplayStyleList.style1:
					case CustomEnums.RiskPSDisplayStyleList.style2:
					case CustomEnums.RiskPSDisplayStyleList.style3:
					{
						priceString =  Risk_PS_contractquantity_str + GetPriceString(price, chartBars)+"    "+extratext;
						break;
					}
						
					case CustomEnums.RiskPSDisplayStyleList.style4:
					case CustomEnums.RiskPSDisplayStyleList.style5:
					case CustomEnums.RiskPSDisplayStyleList.style6:
					{
						priceString =  Risk_PS_contractquantity_str +"    "+ GetPriceString(price, chartBars)+"    "+extratext;
						break;
					}
						
					case CustomEnums.RiskPSDisplayStyleList.style7:
					case CustomEnums.RiskPSDisplayStyleList.style8:
					case CustomEnums.RiskPSDisplayStyleList.style9:
					case CustomEnums.RiskPSDisplayStyleList.style10:
					case CustomEnums.RiskPSDisplayStyleList.style11:
					case CustomEnums.RiskPSDisplayStyleList.style12:
					case CustomEnums.RiskPSDisplayStyleList.style13:
					case CustomEnums.RiskPSDisplayStyleList.style14:
					case CustomEnums.RiskPSDisplayStyleList.style15:
					{
						priceString = GetPriceString(price, chartBars) +"    "+extratext +"    "+ Risk_PS_contractquantity_str;
						break;
					}
				}
			}
			else
				priceString = GetPriceString(price, chartBars) + "    "+extratext;


			
			Stroke color;
			textleftPoint	= RiskAnchor.GetPoint(chartControl, chartPanel, chartScale).X;
			textRightPoint	= EntryAnchor.GetPoint(chartControl, chartPanel, chartScale).X;

			if (anchor == RewardAnchor1)		color = TargetLine1Stroke;
			else
			if (anchor == RewardAnchor2)		color = TargetLine2Stroke;
			else
			if (anchor == RewardAnchor3)		color = TargetLine3Stroke;
			else
			if (anchor == RewardAnchor4)		color = TargetLine4Stroke;
			else if (anchor == RiskAnchor)	color = StopLineStroke;
			else if (anchor == EntryAnchor)	color = EntryLineStroke;
			else							color = AnchorLineStroke;

			SimpleFont						wpfFont		= chartControl.Properties.LabelFont ?? new SimpleFont();
			SharpDX.DirectWrite.TextFormat	textFormat	= wpfFont.ToDirectWriteTextFormat();
			textFormat.TextAlignment					= SharpDX.DirectWrite.TextAlignment.Leading;
			textFormat.WordWrapping						= SharpDX.DirectWrite.WordWrapping.NoWrap;
			SharpDX.DirectWrite.TextLayout textLayout = new SharpDX.DirectWrite.TextLayout(Core.Globals.DirectWriteFactory, priceString, textFormat, chartPanel.H, textFormat.FontSize);

			if (RiskAnchor.Time <= EntryAnchor.Time)
			{
				if(!IsExtendedLinesLeft && !IsExtendedLinesRight)
					switch(TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textleftPoint;								break;
						case TextLocation.InsideRight	: point.X = textRightPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = textleftPoint;								break;
						case TextLocation.ExtremeRight	: point.X = textRightPoint - textLayout.Metrics.Width;	break;
					}
				else if (IsExtendedLinesLeft && !IsExtendedLinesRight)
					switch (TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textleftPoint;								break;
						case TextLocation.InsideRight	: point.X = textRightPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = chartPanel.X;								break;
						case TextLocation.ExtremeRight	: point.X = textRightPoint - textLayout.Metrics.Width;	break;
					}
				else if (!IsExtendedLinesLeft && IsExtendedLinesRight)
					switch (TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textleftPoint;								break;
						case TextLocation.InsideRight	: point.X = textRightPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = textleftPoint;								break;
						case TextLocation.ExtremeRight	: point.X = chartPanel.W - textLayout.Metrics.Width;	break;
					}
				else if (IsExtendedLinesLeft && IsExtendedLinesRight)
					switch (TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textleftPoint;								break;
						case TextLocation.InsideRight	: point.X = textRightPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeRight	: point.X = chartPanel.W - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = chartPanel.X;								break;
					}
			}
			else if (RiskAnchor.Time >= EntryAnchor.Time)
				if (!IsExtendedLinesLeft && !IsExtendedLinesRight)
				{
					switch (TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textRightPoint;								break;
						case TextLocation.InsideRight	: point.X = textleftPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = textRightPoint;								break;
						case TextLocation.ExtremeRight	: point.X = textleftPoint - textLayout.Metrics.Width;	break;
					}
				}
				else if (IsExtendedLinesLeft && !IsExtendedLinesRight)
					switch (TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textRightPoint;								break;
						case TextLocation.InsideRight	: point.X = textleftPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = chartPanel.X;								break;
						case TextLocation.ExtremeRight	: point.X = textleftPoint - textLayout.Metrics.Width;	break;
					}
				else if (!IsExtendedLinesLeft && IsExtendedLinesRight)
					switch (TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textRightPoint;								break;
						case TextLocation.InsideRight	: point.X = textleftPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = textRightPoint;								break;
						case TextLocation.ExtremeRight	: point.X = chartPanel.W - textLayout.Metrics.Width;	break;
					}
				else if (IsExtendedLinesLeft && IsExtendedLinesRight)
					switch (TextAlignment)
					{
						case TextLocation.InsideLeft	: point.X = textRightPoint;								break;
						case TextLocation.InsideRight	: point.X = textleftPoint - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeRight	: point.X = chartPanel.W - textLayout.Metrics.Width;	break;
						case TextLocation.ExtremeLeft	: point.X = chartPanel.X;								break;
					}

			RenderTarget.DrawTextLayout(new SharpDX.Vector2((float)point.X, (float)point.Y), textLayout, color.BrushDX, SharpDX.Direct2D1.DrawTextOptions.NoSnap);
		}

		public override IEnumerable<AlertConditionItem> GetAlertConditionItems()
		{
			return Anchors.Select(anchor => new AlertConditionItem	{
																		Name					= anchor.DisplayName,
																		ShouldOnlyDisplayName	= true,
																		Tag						= anchor
																	});
		}

		public override Cursor GetCursor(ChartControl chartControl, ChartPanel chartPanel, ChartScale chartScale, Point point)
		{
			switch (DrawingState)
			{
				case DrawingState.Building:	return Cursors.Pen;
				case DrawingState.Moving:	return IsLocked ? Cursors.No : Cursors.SizeAll;
				case DrawingState.Editing:	return IsLocked ? Cursors.No : (editingAnchor == EntryAnchor ? Cursors.SizeNESW : Cursors.SizeNWSE);
				default:
					// draw move cursor if cursor is near line path anywhere
					Point entryAnchorPixelPoint	= EntryAnchor.GetPoint(chartControl, chartPanel, chartScale);

					// see if we are near an anchor right away. this is is cheap so no big deal to do often
					ChartAnchor closest = GetClosestAnchor(chartControl, chartPanel, chartScale, cursorSensitivity, point);

					if (closest != null)
						return IsLocked ? Cursors.Arrow : (closest == EntryAnchor ? Cursors.SizeNESW : Cursors.SizeNWSE);

					Point	stopAnchorPixelPoint	= RiskAnchor.GetPoint(chartControl, chartPanel, chartScale);
					Vector	anchorsVector			= stopAnchorPixelPoint - entryAnchorPixelPoint;
	
					// see if the mouse is along one of our lines for moving
					if (MathHelper.IsPointAlongVector(point, entryAnchorPixelPoint, anchorsVector, cursorSensitivity))
						return IsLocked ? Cursors.Arrow : Cursors.SizeAll;

					if (!DrawTarget) 
						return null;

					Point	targetPoint			= RewardAnchor1.GetPoint(chartControl, chartPanel, chartScale);
					Vector	targetToEntryVector	= targetPoint - entryAnchorPixelPoint;
					return MathHelper.IsPointAlongVector(point, entryAnchorPixelPoint, targetToEntryVector, cursorSensitivity) ? (IsLocked ? Cursors.Arrow : Cursors.SizeAll) : null;
			}
		}


		
		private string GetPriceString(double price, ChartBars chartBars)
		{
			string priceString;
			double yValueEntry	= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
			double tickSize		= AttachedTo.Instrument.MasterInstrument.TickSize;
			double pointValue	= AttachedTo.Instrument.MasterInstrument.PointValue;
			switch (DisplayUnit)
			{
				case ValueUnit.Currency:
					if (AttachedTo.Instrument.MasterInstrument.InstrumentType == InstrumentType.Forex)
					{
						priceString = price > yValueEntry ?
							Core.Globals.FormatCurrency(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(price - yValueEntry) / tickSize * (tickSize * pointValue * Account.All[0].ForexLotSize)) :
							Core.Globals.FormatCurrency(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(yValueEntry - price) / tickSize * (tickSize * pointValue * Account.All[0].ForexLotSize));
					}
					else
					{
						priceString = price > yValueEntry ?
							Core.Globals.FormatCurrency(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(price - yValueEntry) / tickSize * (tickSize * pointValue)) :
							Core.Globals.FormatCurrency(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(yValueEntry - price) / tickSize * (tickSize * pointValue));
					}
					break;
				case ValueUnit.Percent:
					priceString = price > yValueEntry ?
						(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(price - yValueEntry) / yValueEntry).ToString("P", Core.Globals.GeneralOptions.CurrentCulture) :
						(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(yValueEntry - price) / yValueEntry).ToString("P", Core.Globals.GeneralOptions.CurrentCulture);
					break;
				case ValueUnit.Ticks:
					priceString = price > yValueEntry ?
						(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(price - yValueEntry) / tickSize).ToString("F0") :
						(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(yValueEntry - price) / tickSize).ToString("F0");
					break;
				case ValueUnit.Pips:
					priceString = price > yValueEntry ?
						(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(price - yValueEntry) / tickSize / 10).ToString("F0") :
						(AttachedTo.Instrument.MasterInstrument.RoundToTickSize(yValueEntry - price) / tickSize / 10).ToString("F0");
					break;
				default:
					priceString = chartBars.Bars.Instrument.MasterInstrument.FormatPrice(price);
					break;
			}
			return priceString;
		}

		public override Point[] GetSelectionPoints(ChartControl chartControl, ChartScale chartScale)
		{
			ChartPanel	chartPanel	= chartControl.ChartPanels[chartScale.PanelIndex];
			Point		entryPoint	= EntryAnchor.GetPoint(chartControl, chartPanel, chartScale);
			Point		stopPoint	= RiskAnchor.GetPoint(chartControl, chartPanel, chartScale);
						
			if (!DrawTarget)
				return new[] { entryPoint, stopPoint };

			Point targetPoint = RewardAnchor1.GetPoint(chartControl, chartPanel, chartScale);
			return new[] { entryPoint, stopPoint, targetPoint };
		}

		public override bool IsAlertConditionTrue(AlertConditionItem conditionItem, Condition condition, ChartAlertValue[] values, ChartControl chartControl, ChartScale chartScale)
		{
			// dig up which anchor we are running on to determine line
			ChartAnchor chartAnchor = conditionItem.Tag as ChartAnchor;
			if (chartAnchor == null)
				return false;

			ChartPanel	chartPanel	= chartControl.ChartPanels[PanelIndex];
			double		alertY		= chartScale.GetYByValue(chartAnchor.Price);
			Point		entryPoint	= EntryAnchor.GetPoint(chartControl, chartPanel, chartScale);
			Point		stopPoint	= RiskAnchor.GetPoint(chartControl, chartPanel, chartScale);
			Point		targetPoint	= RewardAnchor1.GetPoint(chartControl, chartPanel, chartScale);
			double		anchorMinX	= DrawTarget ? new[] { entryPoint.X, stopPoint.X, targetPoint.X }.Min() : new[] {entryPoint.X, stopPoint.X}.Min(); 
			double		anchorMaxX	= DrawTarget ? new[] { entryPoint.X, stopPoint.X, targetPoint.X }.Max() : new[] {entryPoint.X, stopPoint.X}.Max();
			double		lineStartX	= IsExtendedLinesLeft ? chartPanel.X : anchorMinX;
			double		lineEndX	= IsExtendedLinesRight ? chartPanel.X + chartPanel.W : anchorMaxX;

			// first thing, if our smallest x is greater than most recent bar, we have nothing to do yet.
			// do not try to check Y because lines could cross through stuff
			double firstBarX = chartControl.GetXByTime(values[0].Time);
			double firstBarY = chartScale.GetYByValue(values[0].Value);
			
			if (lineEndX < firstBarX) // bars passed our drawing tool
				return false;

			Point lineStartPoint	= new Point(lineStartX, alertY);
			Point lineEndPoint		= new Point(lineEndX, alertY);

			Point barPoint = new Point(firstBarX, firstBarY);
			// NOTE: 'left / right' is relative to if line was vertical. it can end up backwards too
			MathHelper.PointLineLocation pointLocation = MathHelper.GetPointLineLocation(lineStartPoint, lineEndPoint, barPoint);
			// for vertical things, think of a vertical line rotated 90 degrees to lay flat, where it's normal vector is 'up'
			switch (condition)
			{
				case Condition.Greater:			return pointLocation == MathHelper.PointLineLocation.LeftOrAbove;
				case Condition.GreaterEqual:	return pointLocation == MathHelper.PointLineLocation.LeftOrAbove || pointLocation == MathHelper.PointLineLocation.DirectlyOnLine;
				case Condition.Less:			return pointLocation == MathHelper.PointLineLocation.RightOrBelow;
				case Condition.LessEqual:		return pointLocation == MathHelper.PointLineLocation.RightOrBelow || pointLocation == MathHelper.PointLineLocation.DirectlyOnLine;
				case Condition.Equals:			return pointLocation == MathHelper.PointLineLocation.DirectlyOnLine;
				case Condition.NotEqual:		return pointLocation != MathHelper.PointLineLocation.DirectlyOnLine;
				case Condition.CrossAbove:
				case Condition.CrossBelow:
					Predicate<ChartAlertValue> predicate = v =>
					{
						double barX = chartControl.GetXByTime(v.Time);
						double barY = chartScale.GetYByValue(v.Value);
						Point stepBarPoint = new Point(barX, barY);
						// NOTE: 'left / right' is relative to if line was vertical. it can end up backwards too
						MathHelper.PointLineLocation ptLocation = MathHelper.GetPointLineLocation(lineStartPoint, lineEndPoint, stepBarPoint);
						if (condition == Condition.CrossAbove)
							return ptLocation == MathHelper.PointLineLocation.LeftOrAbove;
						return ptLocation == MathHelper.PointLineLocation.RightOrBelow;
					};
					return MathHelper.DidPredicateCross(values, predicate);
			}
			return false;
		}

		public override bool IsVisibleOnChart(ChartControl chartControl, ChartScale chartScale, DateTime firstTimeOnChart, DateTime lastTimeOnChart)
		{
			return DrawingState == DrawingState.Building || Anchors.Any(a => a.Time >= firstTimeOnChart && a.Time <= lastTimeOnChart);
		}

		public override void OnCalculateMinMax()
		{
			// It is important to set MinValue and MaxValue to the min/max Y values your drawing tool uses if you want it to support auto scale
			MinValue = double.MaxValue;
			MaxValue = double.MinValue;

			if (!IsVisible)
				return;

			// return min/max values only if something has been actually drawn
			if (Anchors.Any(a => !a.IsEditing))
				foreach (ChartAnchor anchor in Anchors)
				{
					if(anchor.DisplayName == RewardAnchor1.DisplayName && !DrawTarget)
						continue;

					MinValue = Math.Min(anchor.Price, MinValue);
					MaxValue = Math.Max(anchor.Price, MaxValue);
				}
		}

		public override void OnMouseDown(ChartControl chartControl, ChartPanel chartPanel, ChartScale chartScale, ChartAnchor dataPoint)
		{
			switch(DrawingState)
			{
				case DrawingState.Building:
					if (EntryAnchor.IsEditing)
					{
						dataPoint.CopyDataValues(EntryAnchor);
						dataPoint.CopyDataValues(RiskAnchor);
						EntryAnchor.IsEditing	= false;
						entryPrice				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
					}
					else if (RiskAnchor.IsEditing)
					{
						dataPoint.CopyDataValues(RiskAnchor);
						RiskAnchor.IsEditing	= false;
						stopPrice				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RiskAnchor.Price);
						SetReward1();
						SetReward2();
						SetReward3();
						SetReward4();
						// we set the anchor for the target after stop mouse down event via SetReward()
						//however we need make sure Time is in view when builiding, but always when SetReward is used..
						RewardAnchor1.Time		= EntryAnchor.Time;
						RewardAnchor1.SlotIndex	= EntryAnchor.SlotIndex;
						RewardAnchor1.IsEditing	= false;

						RewardAnchor2.Time		= EntryAnchor.Time;
						RewardAnchor2.SlotIndex	= EntryAnchor.SlotIndex;
						RewardAnchor2.IsEditing	= false;

						RewardAnchor3.Time		= EntryAnchor.Time;
						RewardAnchor3.SlotIndex	= EntryAnchor.SlotIndex;
						RewardAnchor3.IsEditing	= false;

						RewardAnchor4.Time		= EntryAnchor.Time;
						RewardAnchor4.SlotIndex	= EntryAnchor.SlotIndex;
						RewardAnchor4.IsEditing	= false;

					}
					// if the anchors are no longer being edited, set the drawing state to normal and unselect the object
					if (!EntryAnchor.IsEditing && !RiskAnchor.IsEditing && !RewardAnchor1.IsEditing)// && !RewardAnchor2.IsEditing && !RewardAnchor3.IsEditing && !RewardAnchor4.IsEditing)
					{
						DrawingState = DrawingState.Normal;
						IsSelected = false;
					}
					break;
				case DrawingState.Normal:
					Point point = dataPoint.GetPoint(chartControl, chartPanel, chartScale);
					//find which anchor has been clicked relative to the mouse point and make whichever anchor now editable
					editingAnchor = GetClosestAnchor(chartControl, chartPanel, chartScale, cursorSensitivity, point);
					if (editingAnchor != null)
					{
						editingAnchor.IsEditing = true;
						DrawingState = DrawingState.Editing;
					}
					else if (GetCursor(chartControl, chartPanel, chartScale, point) == null)
						IsSelected = false; // missed
					else
					// didnt click an anchor but on a line so start moving
						DrawingState = DrawingState.Moving;
					break;
			}
		}

		public override void OnMouseMove(ChartControl chartControl, ChartPanel chartPanel, ChartScale chartScale, ChartAnchor dataPoint)
		{
			if (IsLocked && DrawingState != DrawingState.Building || !IsVisible)
				return;

			if (DrawingState == DrawingState.Building)
			{
				if (EntryAnchor.IsEditing)
				{
					dataPoint.CopyDataValues(EntryAnchor);
				}
				else if (RiskAnchor.IsEditing)
				{
					dataPoint.CopyDataValues(RiskAnchor);
				}
				else if (RewardAnchor1.IsEditing)
				{
					dataPoint.CopyDataValues(RewardAnchor1);
					dataPoint.CopyDataValues(RewardAnchor2);
					dataPoint.CopyDataValues(RewardAnchor3);
					dataPoint.CopyDataValues(RewardAnchor4);
				}
				
				Update_Risk_PS();
			}
			else if (DrawingState == DrawingState.Editing && editingAnchor != null)
			{
				dataPoint.CopyDataValues(editingAnchor);
				if (editingAnchor != EntryAnchor)
				{
					if (editingAnchor != RewardAnchor1 && Ratio1.ApproxCompare(0) != 0)
					{
						SetReward1();
						SetReward2();
						SetReward3();
						SetReward4();
					}
					else if (Ratio1.ApproxCompare(0) != 0)
					{
						SetRisk();
						SetReward1();
						SetReward2();
						SetReward3();
						SetReward4();
					}
				}
				else
				if (editingAnchor == EntryAnchor)
				{
					SetReward1();
					SetReward2();
					SetReward3();
					SetReward4();
				}
				
				Update_Risk_PS();
			}
			else if (DrawingState == DrawingState.Moving)
			{
				foreach (ChartAnchor anchor in Anchors)
					anchor.MoveAnchor(InitialMouseDownAnchor, dataPoint, chartControl, chartPanel, chartScale, this);
				
					SetReward2();
					SetReward3();
					SetReward4();
			}
			
			entryPrice	= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
			stopPrice	= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RiskAnchor.Price);
			targetPrice	= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RewardAnchor1.Price);
			targetPrice2 = AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RewardAnchor2.Price);
			targetPrice3 = AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RewardAnchor3.Price);
			targetPrice4 = AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RewardAnchor4.Price);
		}

		public override void OnMouseUp(ChartControl chartControl, ChartPanel chartPanel, ChartScale chartScale, ChartAnchor dataPoint)
		{
			//don't set anchors until we're done drawing
			if (DrawingState == DrawingState.Building)
				return;
			
			//set the drawing state back to normal when mouse is relased
			if (DrawingState == DrawingState.Editing || DrawingState == DrawingState.Moving)
				DrawingState = DrawingState.Normal;
			if (editingAnchor != null)
			{
				if (editingAnchor == EntryAnchor)
				{
					SetReward1();
					SetReward2();
					SetReward3();
					SetReward4();
					if (Ratio1.ApproxCompare(0) != 0)	
						SetRisk();
					if (Ratio2.ApproxCompare(0) != 0)	
						SetRisk();
					if (Ratio3.ApproxCompare(0) != 0)	
						SetRisk();
					if (Ratio4.ApproxCompare(0) != 0)	
						SetRisk();
				}
				editingAnchor.IsEditing = false;
			}
			editingAnchor = null;
		}

		public override void OnRender(ChartControl chartControl, ChartScale chartScale)
		{
			if (!IsVisible)
				return;
			if (Anchors.All(a => a.IsEditing))
				return;
		
			// this will be true right away to fix a restoral issue, so check if we really want to set reward
			if (needsRatioUpdate && DrawTarget)
			{
				SetReward1();
				SetReward2();
				SetReward3();
				SetReward4();
			}
			
			if (bShow_PS && Risk_PS_contractquantity == 0)
				Update_Risk_PS();
			
			ChartPanel chartPanel	= chartControl.ChartPanels[PanelIndex];
			Point entryPoint		= EntryAnchor.GetPoint(chartControl, chartPanel, chartScale);
			Point stopPoint			= RiskAnchor.GetPoint(chartControl, chartPanel, chartScale);
			Point targetPoint		= RewardAnchor1.GetPoint(chartControl, chartPanel, chartScale);
			Point targetPoint2		= RewardAnchor2.GetPoint(chartControl, chartPanel, chartScale);
			Point targetPoint3		= RewardAnchor3.GetPoint(chartControl, chartPanel, chartScale);
			Point targetPoint4		= RewardAnchor4.GetPoint(chartControl, chartPanel, chartScale);
						
			AnchorLineStroke.RenderTarget	= RenderTarget;
			EntryLineStroke.RenderTarget	= RenderTarget;
			StopLineStroke.RenderTarget		= RenderTarget;

			// first of all, turn on anti-aliasing to smooth out our line
			RenderTarget.AntialiasMode	= SharpDX.Direct2D1.AntialiasMode.PerPrimitive;
			RenderTarget.DrawLine(entryPoint.ToVector2(), stopPoint.ToVector2(), AnchorLineStroke.BrushDX, AnchorLineStroke.Width, AnchorLineStroke.StrokeStyle);
	
			double	anchorMinX	= DrawTarget ? new[] { entryPoint.X, stopPoint.X, targetPoint.X }.Min() : new[] {entryPoint.X, stopPoint.X}.Min(); 
			double	anchorMaxX	= DrawTarget ? new[] { entryPoint.X, stopPoint.X, targetPoint.X }.Max() : new[] {entryPoint.X, stopPoint.X}.Max();
			double	lineStartX	= IsExtendedLinesLeft ? chartPanel.X : anchorMinX;
			double	lineEndX	= IsExtendedLinesRight ? chartPanel.X + chartPanel.W : anchorMaxX;

			SharpDX.Vector2 entryStartVector	= new SharpDX.Vector2((float)lineStartX, (float)entryPoint.Y);
			SharpDX.Vector2 entryEndVector		= new SharpDX.Vector2((float)lineEndX, (float)entryPoint.Y);
			SharpDX.Vector2 stopStartVector		= new SharpDX.Vector2((float)lineStartX, (float)stopPoint.Y);
			SharpDX.Vector2 stopEndVector		= new SharpDX.Vector2((float)lineEndX, (float)stopPoint.Y);
			
			// don't try and draw the target stuff until we have calculated the target
			SharpDX.Direct2D1.Brush tmpBrush = IsInHitTest ? chartControl.SelectionBrush : AnchorLineStroke.BrushDX;
			if (DrawTarget && Ratio1 >0)
			{	
				AnchorLineStroke.RenderTarget 	= RenderTarget;
				RenderTarget.DrawLine(entryPoint.ToVector2(), targetPoint.ToVector2(), tmpBrush, AnchorLineStroke.Width, AnchorLineStroke.StrokeStyle);
				
				TargetLine1Stroke.RenderTarget		= RenderTarget;
				SharpDX.Vector2 targetStartVector	= new SharpDX.Vector2((float)lineStartX, (float)targetPoint.Y);
				SharpDX.Vector2 targetEndVector		= new SharpDX.Vector2((float)lineEndX, (float)targetPoint.Y);

				tmpBrush = IsInHitTest ? chartControl.SelectionBrush : TargetLine1Stroke.BrushDX;
				RenderTarget.DrawLine(targetStartVector, targetEndVector, tmpBrush, TargetLine1Stroke.Width, TargetLine1Stroke.StrokeStyle);
				DrawPriceText(RewardAnchor1, targetPoint, targetPrice, chartControl, chartPanel, chartScale, "T1 (1:"+Ratio1+")");
			}
			if (DrawTarget && Ratio2 >0 && Ratio2>Ratio1)
			{	
				TargetLine2Stroke.RenderTarget		= RenderTarget;
				SharpDX.Vector2 targetStartVector	= new SharpDX.Vector2((float)lineStartX, (float)targetPoint2.Y);
				SharpDX.Vector2 targetEndVector		= new SharpDX.Vector2((float)lineEndX, (float)targetPoint2.Y);

				tmpBrush = IsInHitTest ? chartControl.SelectionBrush : TargetLine2Stroke.BrushDX;
				RenderTarget.DrawLine(targetStartVector, targetEndVector, tmpBrush, TargetLine2Stroke.Width, TargetLine2Stroke.StrokeStyle);
				DrawPriceText(RewardAnchor2, targetPoint2, targetPrice2, chartControl, chartPanel, chartScale, "T2 (1:"+Ratio2+")");
			}
			if (DrawTarget && Ratio3 >0 && Ratio3>Ratio2)
			{	
				TargetLine3Stroke.RenderTarget		= RenderTarget;
				SharpDX.Vector2 targetStartVector	= new SharpDX.Vector2((float)lineStartX, (float)targetPoint3.Y);
				SharpDX.Vector2 targetEndVector		= new SharpDX.Vector2((float)lineEndX, (float)targetPoint3.Y);

				tmpBrush = IsInHitTest ? chartControl.SelectionBrush : TargetLine3Stroke.BrushDX;
				RenderTarget.DrawLine(targetStartVector, targetEndVector, tmpBrush, TargetLine3Stroke.Width, TargetLine3Stroke.StrokeStyle);
				DrawPriceText(RewardAnchor3, targetPoint3, targetPrice3, chartControl, chartPanel, chartScale, "T3 (1:"+Ratio3+")");
			}
			if (DrawTarget && Ratio4 >0 && Ratio4>Ratio3)
			{	
				TargetLine4Stroke.RenderTarget		= RenderTarget;
				SharpDX.Vector2 targetStartVector	= new SharpDX.Vector2((float)lineStartX, (float)targetPoint4.Y);
				SharpDX.Vector2 targetEndVector		= new SharpDX.Vector2((float)lineEndX, (float)targetPoint4.Y);

				tmpBrush = IsInHitTest ? chartControl.SelectionBrush : TargetLine4Stroke.BrushDX;
				RenderTarget.DrawLine(targetStartVector, targetEndVector, tmpBrush, TargetLine4Stroke.Width, TargetLine4Stroke.StrokeStyle);
				DrawPriceText(RewardAnchor4, targetPoint4, targetPrice4, chartControl, chartPanel, chartScale, "T4 (1:"+Ratio4+")");
			}

			
			tmpBrush = IsInHitTest ? chartControl.SelectionBrush : EntryLineStroke.BrushDX;
			RenderTarget.DrawLine(entryStartVector, entryEndVector, tmpBrush, EntryLineStroke.Width, EntryLineStroke.StrokeStyle);
			DrawPriceText(EntryAnchor, entryPoint, entryPrice, chartControl, chartPanel, chartScale, "ENTRY");

			tmpBrush = IsInHitTest ? chartControl.SelectionBrush : StopLineStroke.BrushDX;
			RenderTarget.DrawLine(stopStartVector, stopEndVector, tmpBrush, StopLineStroke.Width, StopLineStroke.StrokeStyle);
			DrawPriceText(RiskAnchor, stopPoint, stopPrice, chartControl, chartPanel, chartScale, "STOP");
		}

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= "leighRisk-Reward Position Sizing Tool";
				Name						= "leighRisk-Reward Position Sizing Tool";
				Ratio1						= 1;
				Ratio2						= 2;
				Ratio3						= 0;
				Ratio4						= 0;
				AnchorLineStroke 			= new Stroke(Brushes.DarkGray,	DashStyleHelper.Solid, 1f, 50);
				EntryLineStroke 			= new Stroke(Brushes.Goldenrod,	DashStyleHelper.Solid, 2f);
				StopLineStroke 				= new Stroke(Brushes.Crimson,	DashStyleHelper.Solid, 2f);
				TargetLine1Stroke 			= new Stroke(Brushes.SeaGreen,	DashStyleHelper.Solid, 2f);
				TargetLine2Stroke 			= new Stroke(Brushes.SeaGreen,	DashStyleHelper.Solid, 2f);
				TargetLine3Stroke 			= new Stroke(Brushes.SeaGreen,	DashStyleHelper.Solid, 2f);
				TargetLine4Stroke 			= new Stroke(Brushes.SeaGreen,	DashStyleHelper.Solid, 2f);
				EntryAnchor					= new ChartAnchor { IsEditing = true, DrawingTool = this };
				RiskAnchor					= new ChartAnchor { IsEditing = true, DrawingTool = this };
				RewardAnchor1				= new ChartAnchor { IsEditing = true, DrawingTool = this };
				RewardAnchor2				= new ChartAnchor { IsEditing = true, DrawingTool = this };
				RewardAnchor3				= new ChartAnchor { IsEditing = true, DrawingTool = this };
				RewardAnchor4				= new ChartAnchor { IsEditing = true, DrawingTool = this };
				EntryAnchor.DisplayName		= Custom.Resource.NinjaScriptDrawingToolRiskRewardAnchorEntry;
				RiskAnchor.DisplayName		= Custom.Resource.NinjaScriptDrawingToolRiskRewardAnchorRisk;
				RewardAnchor1.DisplayName	= Custom.Resource.NinjaScriptDrawingToolRiskRewardAnchorReward;
				RewardAnchor2.DisplayName	= "Reward 2";
				RewardAnchor3.DisplayName	= "Reward 3";
				RewardAnchor4.DisplayName	= "Reward 4";
				bShowLabels = true;
				
				bShow_PS = false;
				Risk_PS_AccountName = "Sim101";
				Risk_PS_Account = Account.All.First(t => t.Name == Risk_PS_AccountName);
				Risk_PS_AccountBalance = Risk_PS_Account.Get(AccountItem.CashValue, Currency.UsDollar);
            	Risk_PS_MaxPercentageOnAccount = 2.0f;
				Risk_PS_Style = CustomEnums.RiskPSDisplayStyleList.style2;
				Risk_PS_contractquantity = 0;
				bShowWarningExceeded = true;
			}
			else if (State == State.Configure)
			{			
				Risk_PS_contractquantity = 0;
				Risk_PS_Account = Account.All.First(t => t.Name == Risk_PS_AccountName);
				Risk_PS_AccountBalance = Risk_PS_Account.Get(AccountItem.CashValue, Currency.UsDollar);
			}
			else if (State == State.Terminated)
			{
				Dispose();
			}
		}
		
		
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void SetReward1()
		{
			if (Anchors == null || AttachedTo == null)
				return;
			
			entryPrice				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
			stopPrice 				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RiskAnchor.Price);
			risk 					= entryPrice - stopPrice;
			reward					= risk * Ratio1;
			targetPrice 			= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(entryPrice + reward);
			
			RewardAnchor1.Price		= targetPrice;
			RewardAnchor1.IsEditing	= false;
		
			needsRatioUpdate		= false;
		}

		[EditorBrowsable(EditorBrowsableState.Never)]
		public void SetReward2()
		{
			if (Anchors == null || AttachedTo == null)
				return;
			
			entryPrice				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
			stopPrice 				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RiskAnchor.Price);
			risk 					= entryPrice - stopPrice;
			reward2					= risk * Ratio2;
			targetPrice2 			= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(entryPrice + reward2);
			
			RewardAnchor2.Price		= targetPrice2;
			RewardAnchor2.IsEditing	= false;
		
			needsRatioUpdate		= false;
		}

		[EditorBrowsable(EditorBrowsableState.Never)]
		public void SetReward3()
		{
			if (Anchors == null || AttachedTo == null)
				return;
			
			entryPrice				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
			stopPrice 				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RiskAnchor.Price);
			risk 					= entryPrice - stopPrice;
			reward3					= risk * Ratio3;
			targetPrice3 			= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(entryPrice + reward3);
			
			RewardAnchor3.Price		= targetPrice3;
			RewardAnchor3.IsEditing	= false;
		
			needsRatioUpdate		= false;
		}

		[EditorBrowsable(EditorBrowsableState.Never)]
		public void SetReward4()
		{
			if (Anchors == null || AttachedTo == null)
				return;
			
			entryPrice				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
			stopPrice 				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RiskAnchor.Price);
			risk 					= entryPrice - stopPrice;
			reward4					= risk * Ratio4;
			targetPrice4 			= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(entryPrice + reward4);
			
			RewardAnchor4.Price		= targetPrice4;
			RewardAnchor4.IsEditing	= false;
		
			needsRatioUpdate		= false;
		}

		[EditorBrowsable(EditorBrowsableState.Never)]
		public void Update_Risk_PS()
		{
			if (!bShow_PS)
				return;
			
            Risk_PS_Account = Account.All.First(t => t.Name == Risk_PS_AccountName);
			Risk_PS_AccountBalance = Risk_PS_Account.Get(AccountItem.CashValue, Currency.UsDollar);
			double tickSize		= AttachedTo.Instrument.MasterInstrument.TickSize;
			double pointValue	= AttachedTo.Instrument.MasterInstrument.PointValue;
			double tickValue	= tickSize*pointValue;
			double totalriskticks = 0;

			if (stopPrice > entryPrice)
			{
				totalriskticks = (AttachedTo.Instrument.MasterInstrument.RoundToTickSize(stopPrice - entryPrice) / (double)tickSize);
			}
			else
			{
				totalriskticks = (AttachedTo.Instrument.MasterInstrument.RoundToTickSize(entryPrice - stopPrice) / (double)tickSize);
			}
			
			double Risk_PS_Dollars = totalriskticks * tickValue;

			if (Risk_PS_MaxPercentageOnAccount <=0)
			{
				Risk_PS_contractquantity_str ="invalid risk % re-enter    ";
				return;
			}
			double check = Math.Floor((float)Risk_PS_MaxPercentageOnAccount/100d*((double)Risk_PS_AccountBalance));
			double check2 = (double)check/(double)Risk_PS_Dollars;
			Risk_PS_contractquantity = (int)Math.Floor(check2);
			
			if (Risk_PS_contractquantity >0)
			{
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style1)
					Risk_PS_contractquantity_str = Risk_PS_contractquantity.ToString("F0")+" contracts @ ";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style2)
					Risk_PS_contractquantity_str = Risk_PS_contractquantity.ToString("F0")+" lots @ ";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style3)
					Risk_PS_contractquantity_str = Risk_PS_contractquantity.ToString("F0")+" @ ";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style4)
					Risk_PS_contractquantity_str = " ("+Risk_PS_contractquantity.ToString("F0")+" contracts)";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style5)
					Risk_PS_contractquantity_str = " ("+Risk_PS_contractquantity.ToString("F0")+" lots)";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style6)
					Risk_PS_contractquantity_str = " ("+Risk_PS_contractquantity.ToString("F0")+")";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style7)
					Risk_PS_contractquantity_str = Risk_PS_contractquantity.ToString("F0")+" contracts";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style8)
					Risk_PS_contractquantity_str = Risk_PS_contractquantity.ToString("F0")+" lots";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style9)
					Risk_PS_contractquantity_str = "contracts "+Risk_PS_contractquantity.ToString("F0");
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style10)
					Risk_PS_contractquantity_str = "lots "+Risk_PS_contractquantity.ToString("F0");
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style11)
					Risk_PS_contractquantity_str = " (contracts "+Risk_PS_contractquantity.ToString("F0")+")";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style12)
					Risk_PS_contractquantity_str = " (lots "+Risk_PS_contractquantity.ToString("F0")+")";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style13)
					Risk_PS_contractquantity_str = " ("+Risk_PS_contractquantity.ToString("F0")+" contracts)";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style14)
					Risk_PS_contractquantity_str = " ("+Risk_PS_contractquantity.ToString("F0")+" lots)";
				else
				if (Risk_PS_Style == CustomEnums.RiskPSDisplayStyleList.style15)
					Risk_PS_contractquantity_str = " ("+Risk_PS_contractquantity.ToString("F0")+")";
			}
			else
			{
				if (bShowWarningExceeded && stopPrice != entryPrice)
					Risk_PS_contractquantity_str ="warning risk exceeded    ";
			}
		}

			
		[EditorBrowsable(EditorBrowsableState.Never)]
		public void SetRisk()
		{
			if (Anchors == null || AttachedTo == null)
				return;

			entryPrice 				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(EntryAnchor.Price);
			targetPrice				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(RewardAnchor1.Price);
		
			reward					= targetPrice - entryPrice;
			risk					= reward / Ratio1;
			stopPrice 				= AttachedTo.Instrument.MasterInstrument.RoundToTickSize(entryPrice - risk);
			
			RiskAnchor.Price 		= stopPrice;
			RiskAnchor.IsEditing	= false;
		
			needsRatioUpdate		= false;
		}
	}

	public static partial class Draw
	{
		private static RiskReward leighRiskRewardPositionSizingCore(NinjaScriptBase owner, string tag,
			bool isAutoScale, 
			int entryBarsAgo, DateTime entryTime, double entryY,
			int stopBarsAgo, DateTime stopTime, double stopY,
			int targetBarsAgo, DateTime targetTime, double targetY,
			double ratio, bool isStop, bool isGlobal, string templateName)
		{
			if (owner == null)
				throw new ArgumentException("owner");

			if (entryBarsAgo == int.MinValue && entryTime == Core.Globals.MinDate)
				throw new ArgumentException("entry value required");

			if (stopBarsAgo == int.MinValue && stopTime == Core.Globals.MinDate &&
				targetBarsAgo == int.MinValue && targetTime == Core.Globals.MinDate)
				throw new ArgumentException("a stop or target value is required");

			if (isGlobal && tag[0] != GlobalDrawingToolManager.GlobalDrawingToolTagPrefix)
				tag = string.Format("{0}{1}", GlobalDrawingToolManager.GlobalDrawingToolTagPrefix, tag);

			RiskReward riskReward = DrawingTool.GetByTagOrNew(owner, typeof(RiskReward), tag, templateName) as RiskReward;
			
			if (riskReward == null)
				return null;

						
			DrawingTool.SetDrawingToolCommonValues(riskReward, tag, isAutoScale, owner, isGlobal);

			// this is a little tricky, we use entry + (stop or target) to calculate the (target or stop) from ratio
			ChartAnchor	entryAnchor		= DrawingTool.CreateChartAnchor(owner, entryBarsAgo, entryTime, entryY);
			ChartAnchor	stopAnchor;
			ChartAnchor	targetAnchor;

			riskReward.Ratio = ratio;
			
			if (isStop)
			{
				stopAnchor = DrawingTool.CreateChartAnchor(owner, stopBarsAgo, stopTime, stopY);
				entryAnchor.CopyDataValues(riskReward.EntryAnchor);
				entryAnchor.CopyDataValues(riskReward.RewardAnchor);
				stopAnchor.CopyDataValues(riskReward.RiskAnchor);
				riskReward.SetReward();
								
			}
			else 
			{
				targetAnchor = DrawingTool.CreateChartAnchor(owner, targetBarsAgo, targetTime, targetY);
				entryAnchor.CopyDataValues(riskReward.EntryAnchor);
				entryAnchor.CopyDataValues(riskReward.RiskAnchor);
				targetAnchor.CopyDataValues(riskReward.RewardAnchor);
				riskReward.SetRisk();
			}
			
			riskReward.SetState(State.Active);
			return riskReward;
		}

		/// <summary>
		/// Draws a risk/reward on a chart.
		/// </summary>
		/// <param name="owner">The hosting NinjaScript object which is calling the draw method</param>
		/// <param name="tag">A user defined unique id used to reference the draw object</param>
		/// <param name="isAutoScale">Determines if the draw object will be included in the y-axis scale</param>
		/// <param name="entryTime">The time where the draw object's entry will be drawn</param>
		/// <param name="entryY">The y value coordinate where the draw object's entry price will be drawn</param>
		/// <param name="endTime">The end time where the draw object will terminate</param>
		/// <param name="endY">The end y value coordinate where the draw object will terminate</param>
		/// <param name="ratio">An int value determining the calculated ratio between the risk or reward based on the entry point</param>
		/// <param name="isStop">A bool value, when true will use the endTime/endBarsAgo and endY to set the stop, and will automatically calculate the target based off the ratio value.</param>
		/// <returns></returns>
		public static RiskReward leighRiskRewardPositionSizing(NinjaScriptBase owner, string tag, bool isAutoScale, DateTime entryTime, double entryY, DateTime endTime, double endY, double ratio, bool isStop)
		{
			return isStop
				? RiskRewardCore(owner, tag, isAutoScale, int.MinValue, entryTime, entryY, int.MinValue, endTime, endY, 0, Core.Globals.MinDate, 0, ratio, true, false, null)
				: RiskRewardCore(owner, tag, isAutoScale, int.MinValue, entryTime, entryY, 0, Core.Globals.MinDate, 0, int.MinValue, endTime, endY, ratio, false, false, null);
		}

		/// <summary>
		/// Draws a risk/reward on a chart.
		/// </summary>
		/// <param name="owner">The hosting NinjaScript object which is calling the draw method</param>
		/// <param name="tag">A user defined unique id used to reference the draw object</param>
		/// <param name="isAutoScale">Determines if the draw object will be included in the y-axis scale</param>
		/// <param name="entryBarsAgo">The starting bar (x axis coordinate) where the draw object's entry will be drawn. For example, a value of 10 would paint the draw object 10 bars back.</param>
		/// <param name="entryY">The y value coordinate where the draw object's entry price will be drawn</param>
		/// <param name="endBarsAgo">The end bar (x axis coordinate) where the draw object will terminate</param>
		/// <param name="endY">The end y value coordinate where the draw object will terminate</param>
		/// <param name="ratio">An int value determining the calculated ratio between the risk or reward based on the entry point</param>
		/// <param name="isStop">A bool value, when true will use the endTime/endBarsAgo and endY to set the stop, and will automatically calculate the target based off the ratio value.</param>
		/// <returns></returns>
		public static RiskReward leighRiskRewardPositionSizing(NinjaScriptBase owner, string tag, bool isAutoScale, int entryBarsAgo, double entryY, int endBarsAgo, double endY, double ratio, bool isStop)
		{
			return isStop
				? RiskRewardCore(owner, tag, isAutoScale, entryBarsAgo, Core.Globals.MinDate, entryY, endBarsAgo, Core.Globals.MinDate, endY, 0, Core.Globals.MinDate, 0, ratio, true, false, null)
				: RiskRewardCore(owner, tag, isAutoScale, entryBarsAgo, Core.Globals.MinDate, entryY, 0, Core.Globals.MinDate, 0, endBarsAgo, Core.Globals.MinDate, endY, ratio, false, false, null);
		}

		/// <summary>
		/// Draws a risk/reward on a chart.
		/// </summary>
		/// <param name="owner">The hosting NinjaScript object which is calling the draw method</param>
		/// <param name="tag">A user defined unique id used to reference the draw object</param>
		/// <param name="isAutoScale">Determines if the draw object will be included in the y-axis scale</param>
		/// <param name="entryTime">The time where the draw object's entry will be drawn</param>
		/// <param name="entryY">The y value coordinate where the draw object's entry price will be drawn</param>
		/// <param name="endTime">The end time where the draw object will terminate</param>
		/// <param name="endY">The end y value coordinate where the draw object will terminate</param>
		/// <param name="ratio">An int value determining the calculated ratio between the risk or reward based on the entry point</param>
		/// <param name="isStop">A bool value, when true will use the endTime/endBarsAgo and endY to set the stop, and will automatically calculate the target based off the ratio value.</param>
		/// <param name="isGlobal">Determines if the draw object will be global across all charts which match the instrument</param>
		/// <param name="templateName">The name of the drawing tool template the object will use to determine various visual properties</param>
		/// <returns></returns>
		public static RiskReward leighRiskRewardPositionSizing(NinjaScriptBase owner, string tag, bool isAutoScale, DateTime entryTime, double entryY, DateTime endTime, double endY, double ratio, bool isStop, bool isGlobal, string templateName)
		{
			return isStop
				? RiskRewardCore(owner, tag, isAutoScale, int.MinValue, entryTime, entryY, int.MinValue, endTime, endY, 0, Core.Globals.MinDate, 0, ratio, true, isGlobal, templateName)
				: RiskRewardCore(owner, tag, isAutoScale, int.MinValue, entryTime, entryY, 0, Core.Globals.MinDate, 0, int.MinValue, endTime, endY, ratio, false, isGlobal, templateName);
		}

		/// <summary>
		/// Draws a risk/reward on a chart.
		/// </summary>
		/// <param name="owner">The hosting NinjaScript object which is calling the draw method</param>
		/// <param name="tag">A user defined unique id used to reference the draw object</param>
		/// <param name="isAutoScale">Determines if the draw object will be included in the y-axis scale</param>
		/// <param name="entryBarsAgo">The starting bar (x axis coordinate) where the draw object's entry will be drawn. For example, a value of 10 would paint the draw object 10 bars back.</param>
		/// <param name="entryY">The y value coordinate where the draw object's entry price will be drawn</param>
		/// <param name="endBarsAgo">The end bar (x axis coordinate) where the draw object will terminate</param>
		/// <param name="endY">The end y value coordinate where the draw object will terminate</param>
		/// <param name="ratio">An int value determining the calculated ratio between the risk or reward based on the entry point</param>
		/// <param name="isStop">A bool value, when true will use the endTime/endBarsAgo and endY to set the stop, and will automatically calculate the target based off the ratio value.</param>
		/// <param name="isGlobal">Determines if the draw object will be global across all charts which match the instrument</param>
		/// <param name="templateName">The name of the drawing tool template the object will use to determine various visual properties</param>
		/// <returns></returns>
		public static RiskReward leighRiskRewardPositionSizing(NinjaScriptBase owner, string tag, bool isAutoScale, int entryBarsAgo, double entryY, int endBarsAgo, double endY, double ratio, bool isStop, bool isGlobal, string templateName)
		{
			return isStop
				? RiskRewardCore(owner, tag, isAutoScale, entryBarsAgo, Core.Globals.MinDate, entryY, endBarsAgo, Core.Globals.MinDate, endY, 0, Core.Globals.MinDate, 0, ratio, true, isGlobal, templateName)
				: RiskRewardCore(owner, tag, isAutoScale, entryBarsAgo, Core.Globals.MinDate, entryY, 0, Core.Globals.MinDate, 0, endBarsAgo, Core.Globals.MinDate, endY, ratio, false, isGlobal, templateName);
		}
	}
}